use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
Brad
CSV
DHCP
Dedupes
ElasticSearch
GeoIP
Lhotsky
Logstash
MaxMind
Suricata
attacks
bin
brad
caddy
cee
context
contexts
contextualizer
crond
debug
decoder
decoders
dhcpd
dictionary
elasticsearch
eris
filterlog
fluentd
geo
hash
iptables
json
lib
log
matcher
matchers
pfSense
pluggable
plugin
postfix
referer
role
schema
schemas
snort
sshd
static
sudo
suricata
syslog
syslog-ng
url
yum
